import json


def simple_message(sender, type="message", target="server", as_string=True, data=None):
    m = {
        "type": type,
        "data": data,
        "from": sender,
        "target": target if isinstance(target, list) else [target],
    }
    if as_string:
        return json.dumps(m)
    return m


def commandmessage(cmd, sender, target="server", as_string=True, *args, **kwargs):
    m = simple_message(
        sender,
        type="cmd",
        as_string=False,
        target=target,
        data={"cmd": cmd, "args": args, "kwargs": kwargs},
    )
    if as_string:
        return json.dumps(m)
    return m


def levelmessage(
    sender, content="", target="server", title="", level="info", as_string=True
):
    m = simple_message(
        sender,
        type="message",
        as_string=False,
        target=target,
        data={"content": content, "title": title, "level": level},
    )
    if as_string:
        return json.dumps(m)
    return m


def datapointmessage(sender, x, y, key, target="server", t=None, as_string=True):
    m = simple_message(
        sender,
        type="data",
        as_string=False,
        data={"key": key, "x": x, "y": y, "t": t},
        target=target,
    )
    if as_string:
        return json.dumps(m)
    return m



import time

import logging
import random
import threading

from websocket_communication_server.messagetemplates import commandmessage
from websocket_communication_server.socketclient import WebSocketClient
from websocket_communication_server.socketserver import SockerServer, SOCKETPORT, connect_to_first_free_port


class Testclass:
    def __init__(self, host, name):
        self.runime = 0
        self.name = name
        self.wsc = WebSocketClient(name, host=host,password="PingPongBall")
        self.wsc.add_cmd_function("pass_ball", self.pass_ball) # registers own pass_ball method to be called when reciving the pass_ball command
        self.pingpong = "ping" if "1" in name else "pong"
        self.opponent = "Player2" if "1" in name else "Player1"

    def pass_ball(self, catch=None): #methods called by the message validator should have default attributes to be valid against "TypeError: pass_ball() missing 1 required positional argument: 'catch'"
        if catch is None:
            print("End of the game because the ball went missing somewhere")
            return
        print(self.pingpong)  # prints ping or pong
        if not catch:
            print("Damn!")  # dont like losing the ball
        time.sleep(1)  # delay in game because it would be way to fast without, maybe the flytime of the ball :)
        self.wsc.write_to_socket(
            commandmessage(
                cmd="pass_ball", # the command to call at the reciver
                sender=self.name, # sender of the command
                target=self.opponent, # reciver
                catch=random.random() < 0.8, # 80% ball catch
            )
        )


if __name__ == "__main__":
    # connects to the firsts free port
    logging.basicConfig(
        level=logging.DEBUG,
        format="%(asctime)s %(filename)s %(lineno)d %(name)s %(levelname)-8s  %(message)s",
        datefmt="(%H:%M:%S)",
    )

    notconnected = True
    socketserver = connect_to_first_free_port(password="PingPongBall",pass_is_cleartext=True)
    print(socketserver.ws_adress)
    print(connect_to_first_free_port().ws_adress)
    threading.Thread(
        target=socketserver.run_forever,daemon=True
    ).start()  # runs server forever in background



    test1 = Testclass(host=socketserver.ws_adress, name="Player1")  # create player 1
    test2 = Testclass(host=socketserver.ws_adress, name="Player2")  # create player 2
    time.sleep(1)  # gives the players enought timw to identify
    test1.pass_ball(True)  # throw the first ball
    try: print(connect_to_first_free_port().ws_adress)
    except Exception as e: print(e)
    for i in range(10):
        time.sleep(1)  # plays 10 seconds
    socketserver.force_stop()
    time.sleep(2)
    try: print(connect_to_first_free_port().ws_adress)
    except Exception as e: print(e)


import asyncio
import binascii
import hashlib
import os

import time

import logging
import websockets

from websocket_communication_server.connection import Connection

SOCKETPORT = 8888


class SockerServer:
    def __init__(self, host="127.0.0.1", port=SOCKETPORT,password=None,pass_is_cleartext=False):
        logging.getLogger("websocket").setLevel(logging.ERROR)
        logging.getLogger("asyncio").setLevel(logging.ERROR)
        logging.getLogger("asyncio.coroutines").setLevel(logging.ERROR)
        logging.getLogger("websockets.server").setLevel(logging.ERROR)
        logging.getLogger("websockets.protocol").setLevel(logging.ERROR)
        self.running=True
        self.all_connections = {}
        self.logger = logging.getLogger("sockerserver_" + str(host) + ":" + str(port))
        self.host = host
        self.port = port
        self.ws_adress = "ws://" + host + ":" + str(port)

        self.loop = asyncio.new_event_loop()
        asyncio.set_event_loop(self.loop)
        ws_serv = websockets.serve(
            self.add_connection, host=host, port=port, max_size=2 ** 25
        )

        self.t0 = time.time()
        self.loop.run_until_complete(ws_serv)
        self.logger.info("Socket created at " + host + ":" + str(port))
        self.ws_server=ws_serv.ws_server
        self.password=None
        if password is not None:
            self.set_password(password,pass_is_cleartext)

    def set_password(self, password, pass_is_cleartext):
        if pass_is_cleartext:
            password = self.hash_password(password)
        self.password=password

    def hash_password(self,password):
        """Hash a password for storing."""
        salt = hashlib.sha256(os.urandom(60)).hexdigest().encode('ascii')
        pwdhash = hashlib.pbkdf2_hmac('sha512', password.encode('utf-8'),
                                      salt, 100000)
        pwdhash = binascii.hexlify(pwdhash)
        return (salt + pwdhash).decode('ascii')

    def verify_password(self, provided_password):
        """Verify a stored password against one provided by user"""
        if self.password is None:
            return True
        salt = self.password[:64]
        stored_password = self.password[64:]
        pwdhash = hashlib.pbkdf2_hmac('sha512',
                                      provided_password.encode('utf-8'),
                                      salt.encode('ascii'),
                                      100000)
        pwdhash = binascii.hexlify(pwdhash).decode('ascii')
        return pwdhash == stored_password

    def send_to_names(self, names, message):
        while "server" in names:
            names.remove("server")
        reached=[]
        for ws, c in self.all_connections.items():
            if c.name in names:
                reached.append(c.name)
                c.sendMsg(message)
        diff=set(names).difference(set(reached))
        if len(diff)>0:
            self.logger.error("targets not found: "+", ".join(diff))


    def send_to_all(self, message):
        for ws, c in self.all_connections.items():
            c.sendMsg(message)

    async def add_connection(self, ws, path):
        if ws not in self.all_connections:
            # from websocket_communication_server.connection import Connection

            self.all_connections[ws] = Connection(ws, self)
            self.all_connections[ws].ask_for_identification()
        connection = self.all_connections[ws]
        try:
            while 1:
                await connection.recive()
        except Exception as e:
            self.logger.error("Conenction Error " + connection.name)
            self.logger.exception(e)
            del self.all_connections[ws]
            del connection

    def run_forever(self):
        self.loop.run_forever()

    def get_www_data_path(self):
        import os
        from websocket_communication_server import www_data
        return os.path.abspath(os.path.dirname(www_data.__file__))

    def force_stop(self):
        self.loop.stop()
        for i in range(10):
            if not self.loop.is_running():
                break
            time.sleep(1)
        self.ws_server.close()




def connect_to_first_free_port(startport=SOCKETPORT,**kwargs):
    notconnected = True
    socketserver = None

    while notconnected:
        try:
            print(startport)
            socketserver = SockerServer(port=startport,**kwargs)
            notconnected = False
        except OSError as e:
            startport += 1
    return socketserver

if __name__ == "__main__":

    logging.basicConfig(
        level=logging.DEBUG,
        format="%(asctime)s %(filename)s %(lineno)d %(name)s %(levelname)-8s  %(message)s",
        datefmt="(%H:%M:%S)",
    )

    socketserver=connect_to_first_free_port(password=None)

    from websocket_communication_server.socketclient import WebSocketClient

    autoclient = WebSocketClient("testclient", host=socketserver.ws_adress)
    socketserver = connect_to_first_free_port()
    print(socketserver.get_www_data_path())
    socketserver.run_forever()